# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar, List, Optional
from typing_extensions import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.v2.billing._custom_pricing_unit import CustomPricingUnit
    from stripe.v2.billing._metered_item import MeteredItem


class RateCardRate(StripeObject):
    """
    A Rate Card Rate represents a single usage-based price within a Rate Card. Each rate binds to one Metered Item and
    defines the pricing structure for that item, including either a flat unit amount or tiered pricing. Rates support
    features like graduated or volume-based tiering, quantity transformations, and custom pricing units.
    """

    OBJECT_NAME: ClassVar[Literal["v2.billing.rate_card_rate"]] = (
        "v2.billing.rate_card_rate"
    )

    class CustomPricingUnitAmount(StripeObject):
        custom_pricing_unit_details: Optional["CustomPricingUnit"]
        """
        The Custom Pricing Unit object.
        """
        id: str
        """
        The id of the custom pricing unit.
        """
        value: str
        """
        The unit value for the custom pricing unit, as a string.
        """

    class Tier(StripeObject):
        flat_amount: Optional[str]
        """
        Price for the entire tier, represented as a decimal string in minor currency units with at most 12 decimal places.
        """
        unit_amount: Optional[str]
        """
        Per-unit price for units included in this tier, represented as a decimal string in minor currency units with at
        most 12 decimal places.
        """
        up_to_decimal: Optional[Decimal]
        """
        Up to and including this quantity is contained in the tier. Only one of `up_to_decimal` and `up_to_inf` may
        be set.
        """
        up_to_inf: Optional[Literal["inf"]]
        """
        No upper bound to this tier. Only one of `up_to_decimal` and `up_to_inf` may be set.
        """
        _field_encodings = {"up_to_decimal": "decimal_string"}

    class TransformQuantity(StripeObject):
        divide_by: int
        """
        Divide usage by this number.
        """
        round: Literal["down", "up"]
        """
        After division, round the result up or down.
        """
        _field_encodings = {"divide_by": "int64_string"}

    created: str
    """
    Timestamp of when the object was created.
    """
    custom_pricing_unit_amount: Optional[CustomPricingUnitAmount]
    """
    The custom pricing unit that this rate binds to.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    metered_item: "MeteredItem"
    """
    A Metered Item represents a billable item whose pricing is based on usage, measured by a meter. You can use rate cards
    to specify the pricing and create subscriptions to these items.
    """
    object: Literal["v2.billing.rate_card_rate"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    rate_card: str
    """
    The ID of the Rate Card it belongs to.
    """
    rate_card_version: str
    """
    The ID of the Rate Card Version it was created on.
    """
    tiering_mode: Optional[Literal["graduated", "volume"]]
    """
    Defines whether the tiering price is graduated or volume-based. In volume-based tiering, the maximum
    quantity within a period determines the per-unit price. In graduated tiering, the pricing changes as the quantity
    grows into new tiers. Can only be set if `tiers` is set.
    """
    tiers: List[Tier]
    """
    Each element represents a pricing tier. Cannot be set if `unit_amount` is provided.
    """
    transform_quantity: Optional[TransformQuantity]
    """
    Apply a transformation to the reported usage or set quantity before computing the amount billed.
    """
    unit_amount: Optional[str]
    """
    The per-unit amount to be charged, represented as a decimal string in minor currency units with at most 12 decimal
    places. Cannot be set if `tiers` is provided.
    """
    _inner_class_types = {
        "custom_pricing_unit_amount": CustomPricingUnitAmount,
        "tiers": Tier,
        "transform_quantity": TransformQuantity,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class TransactionLineItem(StripeObject):
    OBJECT_NAME: ClassVar[Literal["tax.transaction_line_item"]] = (
        "tax.transaction_line_item"
    )

    class PerformanceLocationDetails(StripeObject):
        class Address(StripeObject):
            city: Optional[str]
            """
            City, district, suburb, town, or village.
            """
            country: str
            """
            Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
            """
            line1: Optional[str]
            """
            Address line 1, such as the street, PO Box, or company name.
            """
            line2: Optional[str]
            """
            Address line 2, such as the apartment, suite, unit, or building.
            """
            postal_code: Optional[str]
            """
            ZIP or postal code.
            """
            state: Optional[str]
            """
            State/province as an [ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2) subdivision code, without country prefix, such as "NY" or "TX".
            """

        address: Address
        _inner_class_types = {"address": Address}

    class Reversal(StripeObject):
        original_line_item: str
        """
        The `id` of the line item to reverse in the original transaction.
        """

    amount: int
    """
    The line item amount in the [smallest currency unit](https://docs.stripe.com/currencies#minor-units). If `tax_behavior=inclusive`, then this amount includes taxes. Otherwise, taxes were calculated on top of this amount.
    """
    amount_tax: int
    """
    The amount of tax calculated for this line item, in the [smallest currency unit](https://docs.stripe.com/currencies#minor-units).
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["tax.transaction_line_item"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    performance_location_details: Optional[PerformanceLocationDetails]
    """
    The address of the location where this line item's event or service takes place. Depending on the [tax code](https://docs.stripe.com/tax/tax-codes), providing a performance location is required, optional, or not supported. Use this to provide the address inline without pre-creating a [TaxLocation](https://docs.stripe.com/api/tax/location) object. Can't be used with `performance_location`.
    """
    product: Optional[str]
    """
    The ID of an existing [Product](https://docs.stripe.com/api/products/object).
    """
    quantity: int
    """
    The number of units of the item being purchased. For reversals, this is the quantity reversed.
    """
    reference: str
    """
    A custom identifier for this line item in the transaction.
    """
    reversal: Optional[Reversal]
    """
    If `type=reversal`, contains information about what was reversed.
    """
    tax_behavior: Literal["exclusive", "inclusive"]
    """
    Specifies whether the `amount` includes taxes. If `tax_behavior=inclusive`, then the amount includes taxes.
    """
    tax_code: str
    """
    The [tax code](https://docs.stripe.com/tax/tax-categories) ID used for this resource.
    """
    type: Literal["reversal", "transaction"]
    """
    If `reversal`, this line item reverses an earlier transaction.
    """
    _inner_class_types = {
        "performance_location_details": PerformanceLocationDetails,
        "reversal": Reversal,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar
from typing_extensions import Literal


class ContractLicensePricingQuantityChange(StripeObject):
    """
    A license pricing quantity change object returned by ListContractLicenseQuantityChanges.
    """

    OBJECT_NAME: ClassVar[
        Literal["v2.billing.contract_license_pricing_quantity_change"]
    ] = "v2.billing.contract_license_pricing_quantity_change"
    created: str
    """
    The timestamp when this quantity change object was created.
    """
    effective_at: str
    """
    The timestamp when this quantity change takes effect.
    """
    id: str
    """
    The ID of the quantity change object.
    """
    license_pricing_id: str
    """
    The ID of the license pricing.
    """
    license_pricing_type: Literal["license_fee", "price"]
    """
    The type of the license pricing.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.billing.contract_license_pricing_quantity_change"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    pricing_line: str
    """
    The ID of the pricing line associated with this quantity change.
    """
    quantity: int
    """
    The quantity at the effective time.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import NotRequired, TypedDict


class ContractListParams(TypedDict):
    customer: NotRequired[str]
    """
    Filter by customer ID.
    """
    limit: NotRequired[int]
    """
    The limit for the number of results per page.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe.v2._amount import Amount as V2AmountResource
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class Contract(StripeObject):
    """
    Main Contract resource representing a comprehensive billing agreement
    """

    OBJECT_NAME: ClassVar[Literal["v2.billing.contract"]] = (
        "v2.billing.contract"
    )

    class BillingSettings(StripeObject):
        class ContractBillingDetails(StripeObject):
            class BillSettingsDetails(StripeObject):
                class Calculation(StripeObject):
                    class Tax(StripeObject):
                        type: Literal["automatic", "manual"]
                        """
                        The type of tax calculation.
                        """

                    tax: Optional[Tax]
                    """
                    Tax calculation settings.
                    """
                    _inner_class_types = {"tax": Tax}

                class Invoice(StripeObject):
                    class TimeUntilDue(StripeObject):
                        interval: Literal["day", "month", "week", "year"]
                        """
                        The interval unit.
                        """
                        interval_count: int
                        """
                        The number of intervals.
                        """

                    time_until_due: Optional[TimeUntilDue]
                    """
                    The number of time units before the invoice is past due.
                    """
                    _inner_class_types = {"time_until_due": TimeUntilDue}

                calculation: Optional[Calculation]
                """
                Calculation settings.
                """
                invoice: Optional[Invoice]
                """
                Invoice settings.
                """
                _inner_class_types = {
                    "calculation": Calculation,
                    "invoice": Invoice,
                }

            class BillingProfileDetails(StripeObject):
                customer: str
                """
                The customer who pays for the contract invoice.
                """
                default_payment_method: Optional[str]
                """
                The default payment method for the contract.
                """

            class CollectionSettingsDetails(StripeObject):
                collection_method: Literal[
                    "charge_automatically", "send_invoice"
                ]
                """
                The collection method.
                """
                payment_method_configuration: Optional[str]
                """
                The payment method configuration.
                """

            bill_settings_details: Optional[BillSettingsDetails]
            """
            The bill settings details.
            """
            billing_profile_details: BillingProfileDetails
            """
            The billing profile details.
            """
            collection_settings_details: CollectionSettingsDetails
            """
            The collection settings details.
            """
            _inner_class_types = {
                "bill_settings_details": BillSettingsDetails,
                "billing_profile_details": BillingProfileDetails,
                "collection_settings_details": CollectionSettingsDetails,
            }

        contract_billing_details: Optional[ContractBillingDetails]
        """
        Billing settings details for the contract.
        """
        _inner_class_types = {
            "contract_billing_details": ContractBillingDetails,
        }

    class ContractLineDetail(StripeObject):
        class ContractLineValueDetails(StripeObject):
            total: str
            """
            Computed sum of all licensed fees. Represented as a decimal string in minor currency units.
            """

        class EndsAt(StripeObject):
            timestamp: str
            """
            The timestamp when the item ends.
            """

        class Override(StripeObject):
            class EndsAt(StripeObject):
                timestamp: str
                """
                The timestamp when the item ends.
                """

            class ServiceAction(StripeObject):
                class Add(StripeObject):
                    class CreditGrant(StripeObject):
                        class Amount(StripeObject):
                            monetary: Optional[V2AmountResource]
                            """
                            The monetary amount of the credit grant. Required if `type` is `monetary`.
                            """
                            type: Literal["monetary"]
                            """
                            The type of the credit grant amount.
                            """

                        class ApplicabilityConfig(StripeObject):
                            class Scope(StripeObject):
                                billable_items: Optional[List[str]]
                                """
                                The billable items to apply the credit grant to.
                                """
                                price_type: Optional[Literal["metered"]]
                                """
                                The price type that credit grants can apply to.
                                """

                            scope: Scope
                            """
                            The applicability scope of the credit grant.
                            """
                            _inner_class_types = {"scope": Scope}

                        class ExpiryConfig(StripeObject):
                            type: Literal["end_of_service_period"]
                            """
                            The type of the expiry configuration.
                            """

                        amount: Amount
                        """
                        The amount of the credit grant.
                        """
                        applicability_config: ApplicabilityConfig
                        """
                        Defines the scope where the credit grant is applicable.
                        """
                        category: Optional[Literal["paid", "promotional"]]
                        """
                        The category of the credit grant.
                        """
                        expiry_config: ExpiryConfig
                        """
                        The expiry configuration for the credit grant.
                        """
                        name: str
                        """
                        A descriptive name.
                        """
                        priority: Optional[int]
                        """
                        The desired priority for applying this credit grant. The highest priority is 0 and the lowest is 100.
                        """
                        _inner_class_types = {
                            "amount": Amount,
                            "applicability_config": ApplicabilityConfig,
                            "expiry_config": ExpiryConfig,
                        }

                    credit_grant: Optional[CreditGrant]
                    """
                    Details for the credit grant. Required if `type` is `credit_grant`.
                    """
                    service_interval: Literal["day", "month", "week", "year"]
                    """
                    The interval for assessing service.
                    """
                    service_interval_count: int
                    """
                    The length of the interval for assessing service.
                    """
                    type: Literal["credit_grant"]
                    """
                    The type of the service action.
                    """
                    _inner_class_types = {"credit_grant": CreditGrant}

                class Replace(StripeObject):
                    class CreditGrant(StripeObject):
                        class Amount(StripeObject):
                            monetary: Optional[V2AmountResource]
                            """
                            The monetary amount of the credit grant. Required if `type` is `monetary`.
                            """
                            type: Literal["monetary"]
                            """
                            The type of the credit grant amount.
                            """

                        class ApplicabilityConfig(StripeObject):
                            class Scope(StripeObject):
                                billable_items: Optional[List[str]]
                                """
                                The billable items to apply the credit grant to.
                                """
                                price_type: Optional[Literal["metered"]]
                                """
                                The price type that credit grants can apply to.
                                """

                            scope: Scope
                            """
                            The applicability scope of the credit grant.
                            """
                            _inner_class_types = {"scope": Scope}

                        class ExpiryConfig(StripeObject):
                            type: Literal["end_of_service_period"]
                            """
                            The type of the expiry configuration.
                            """

                        amount: Amount
                        """
                        The amount of the credit grant.
                        """
                        applicability_config: ApplicabilityConfig
                        """
                        Defines the scope where the credit grant is applicable.
                        """
                        category: Optional[Literal["paid", "promotional"]]
                        """
                        The category of the credit grant.
                        """
                        expiry_config: ExpiryConfig
                        """
                        The expiry configuration for the credit grant.
                        """
                        name: str
                        """
                        A descriptive name.
                        """
                        priority: Optional[int]
                        """
                        The desired priority for applying this credit grant. The highest priority is 0 and the lowest is 100.
                        """
                        _inner_class_types = {
                            "amount": Amount,
                            "applicability_config": ApplicabilityConfig,
                            "expiry_config": ExpiryConfig,
                        }

                    credit_grant: Optional[CreditGrant]
                    """
                    Details for the credit grant. Required if `type` is `credit_grant`.
                    """
                    id: Optional[str]
                    """
                    The ID of the service action to replace.
                    """
                    lookup_key: Optional[str]
                    """
                    The lookup key for the service action to replace.
                    """
                    service_interval: Literal["day", "month", "week", "year"]
                    """
                    The interval for assessing service.
                    """
                    service_interval_count: int
                    """
                    The length of the interval for assessing service.
                    """
                    type: Literal["credit_grant"]
                    """
                    The type of the service action.
                    """
                    _inner_class_types = {"credit_grant": CreditGrant}

                add: Optional[Add]
                """
                Parameters for adding a new service action.
                """
                replace: Optional[Replace]
                """
                Parameters for replacing an existing service action.
                """
                type: Literal["add", "replace"]
                """
                The type of service action override.
                """
                _inner_class_types = {"add": Add, "replace": Replace}

            class StartsAt(StripeObject):
                timestamp: str
                """
                The timestamp when the item starts.
                """

            ends_at: EndsAt
            """
            Timestamp to indicate when the override ends.
            """
            service_action: Optional[ServiceAction]
            """
            Service action override details.
            """
            starts_at: StartsAt
            """
            Timestamp to indicate when the override starts.
            """
            type: Literal["service_action"]
            """
            The type of the override.
            """
            _inner_class_types = {
                "ends_at": EndsAt,
                "service_action": ServiceAction,
                "starts_at": StartsAt,
            }

        class Pricing(StripeObject):
            pass

        class StartsAt(StripeObject):
            timestamp: str
            """
            The timestamp when the item starts.
            """

        contract_line: str
        """
        The ID of the contract line.
        """
        contract_line_value_details: ContractLineValueDetails
        """
        The computed value details for the contract line.
        """
        created: str
        """
        Timestamp of when the object was created.
        """
        ends_at: EndsAt
        """
        Timestamp to indicate when the contract line ends.
        """
        metadata: Optional[UntypedStripeObject[str]]
        """
        Set of key-value pairs that you can attach to an object.
        """
        overrides: List[Override]
        """
        List of overrides applied to the contract line.
        """
        pricing: Pricing
        """
        The pricing configuration for the contract line.
        """
        starts_at: StartsAt
        """
        Timestamp to indicate when the contract line starts.
        """
        _inner_class_types = {
            "contract_line_value_details": ContractLineValueDetails,
            "ends_at": EndsAt,
            "overrides": Override,
            "pricing": Pricing,
            "starts_at": StartsAt,
        }

    class ContractValueDetails(StripeObject):
        total: str
        """
        The total value represented as a decimal string in minor currency units.
        """

    class LicenseQuantity(StripeObject):
        license_pricing_id: str
        """
        The ID of the license pricing.
        """
        license_pricing_type: Literal["license_fee", "price"]
        """
        The type of the license pricing.
        """
        pricing_line: str
        """
        The ID of the pricing line associated with this license quantity.
        """
        quantity: int
        """
        The current quantity of the license.
        """

    class OneTimeFee(StripeObject):
        class BillSchedule(StripeObject):
            class BillAt(StripeObject):
                timestamp: Optional[str]
                """
                The datetime at which the entry will be billed. Set when `type` is `datetime`.
                """
                type: Literal["contract_start", "datetime"]
                """
                The type of the bill_at.
                """

            bill_at: BillAt
            """
            When this entry will be billed.
            """
            value: int
            """
            The amount to bill for this entry, in the smallest currency unit.
            """
            _inner_class_types = {"bill_at": BillAt}
            _field_encodings = {"value": "int64_string"}

        class ProductDetails(StripeObject):
            product: str
            """
            The ID of the v1 Product.
            """

        bill_schedule: List[BillSchedule]
        """
        The resolved bill schedule for the fee.
        """
        billable_item_type: Literal["product"]
        """
        The type of billable item that this fee references.
        """
        product_details: Optional[ProductDetails]
        """
        Details for a product billable target. Set when `billable_item_type` is `product`.
        """
        _inner_class_types = {
            "bill_schedule": BillSchedule,
            "product_details": ProductDetails,
        }

    class PricingLine(StripeObject):
        class EndsAt(StripeObject):
            timestamp: str
            """
            The timestamp when the item ends.
            """

        class Pricing(StripeObject):
            class PriceDetails(StripeObject):
                price: str
                """
                The ID of the V1 price.
                """

            price_details: Optional[PriceDetails]
            """
            V1 price details. Present when `type` is `price`.
            """
            type: Literal["price"]
            """
            The type of pricing.
            """
            _inner_class_types = {"price_details": PriceDetails}

        class StartsAt(StripeObject):
            timestamp: str
            """
            The timestamp when the item starts.
            """

        ends_at: EndsAt
        """
        Resolved timestamp when the pricing line ends.
        """
        lookup_key: Optional[str]
        """
        The user-provided lookup key for the pricing line.
        """
        metadata: Optional[UntypedStripeObject[str]]
        """
        Set of key-value pairs that you can attach to an object.
        """
        pricing: Pricing
        """
        The pricing configuration for the pricing line.
        """
        pricing_line: str
        """
        The ID of the pricing line.
        """
        starts_at: StartsAt
        """
        Resolved timestamp when the pricing line starts.
        """
        _inner_class_types = {
            "ends_at": EndsAt,
            "pricing": Pricing,
            "starts_at": StartsAt,
        }

    class PricingOverride(StripeObject):
        class EndsAt(StripeObject):
            timestamp: str
            """
            The timestamp when the item ends.
            """

        class Multiplier(StripeObject):
            class Criterion(StripeObject):
                class MetadataCondition(StripeObject):
                    class AllOf(StripeObject):
                        key: str
                        """
                        The metadata key.
                        """
                        value: str
                        """
                        The metadata value.
                        """

                    all_of: List[AllOf]
                    """
                    All of these key-value conditions must match.
                    """
                    _inner_class_types = {"all_of": AllOf}

                billable_item_ids: List[str]
                """
                Filter by billable item IDs.
                """
                billable_item_lookup_keys: List[str]
                """
                Filter by billable item lookup keys.
                """
                billable_item_types: List[Literal["licensed", "metered"]]
                """
                Filter by billable item type.
                """
                metadata_conditions: List[MetadataCondition]
                """
                Filter by metadata conditions.
                """
                rate_card_ids: List[str]
                """
                Filter by rate card IDs. Only applicable for `multiplier` overrides.
                """
                type: Literal["exclude", "include"]
                """
                Whether to include or exclude items matching these criteria.
                """
                _inner_class_types = {"metadata_conditions": MetadataCondition}

            criteria: List[Criterion]
            """
            Criteria determining which rates the multiplier applies to.
            """
            factor: str
            """
            The multiplier factor, represented as a decimal string. e.g. "0.8" for a 20% reduction.
            """
            _inner_class_types = {"criteria": Criterion}

        class OverwritePrice(StripeObject):
            class Tier(StripeObject):
                flat_amount: Optional[str]
                """
                Price for the entire tier, represented as a decimal string in minor currency units.
                """
                unit_amount: Optional[str]
                """
                Per-unit price for units included in this tier, represented as a decimal string in minor currency units.
                """
                up_to_decimal: Optional[Decimal]
                """
                Up to and including this quantity will be contained in the tier.
                """
                up_to_inf: Optional[Literal["inf"]]
                """
                No upper bound to this tier.
                """
                _field_encodings = {"up_to_decimal": "decimal_string"}

            price: str
            """
            The ID of the V1 price to overwrite.
            """
            tiering_mode: Optional[Literal["graduated", "volume"]]
            """
            Defines whether the tiered price should be graduated or volume-based.
            """
            tiers: List[Tier]
            """
            Each element represents a pricing tier.
            """
            unit_amount: Optional[str]
            """
            The per-unit amount to be charged, represented as a decimal string in minor currency units.
            """
            _inner_class_types = {"tiers": Tier}

        class StartsAt(StripeObject):
            timestamp: str
            """
            The timestamp when the item starts.
            """

        ends_at: EndsAt
        """
        Resolved timestamp when the pricing override ends.
        """
        lookup_key: Optional[str]
        """
        The user-provided lookup key for the pricing override.
        """
        multiplier: Optional[Multiplier]
        """
        Details for a multiplier override.
        """
        overwrite_price: Optional[OverwritePrice]
        """
        Details for an overwrite_price override.
        """
        pricing_override: str
        """
        The ID of the pricing override.
        """
        priority: int
        """
        The priority of this override relative to others. Lower number = higher priority.
        """
        starts_at: StartsAt
        """
        Resolved timestamp when the pricing override starts.
        """
        type: Literal["multiplier", "overwrite_price"]
        """
        The type of pricing override.
        """
        _inner_class_types = {
            "ends_at": EndsAt,
            "multiplier": Multiplier,
            "overwrite_price": OverwritePrice,
            "starts_at": StartsAt,
        }

    class StatusDetails(StripeObject):
        class Active(StripeObject):
            activated_at: str
            """
            The timestamp when the contract was activated.
            """

        class Canceled(StripeObject):
            canceled_at: str
            """
            The timestamp when the contract was canceled.
            """

        active: Optional[Active]
        """
        Details of the active contract status.
        """
        canceled: Optional[Canceled]
        """
        Details of the canceled contract status.
        """
        _inner_class_types = {"active": Active, "canceled": Canceled}

    billing_settings: Optional[BillingSettings]
    """
    The billing settings for the contract.
    """
    contract_line_details: List[ContractLineDetail]
    """
    The contract line details of the contract. Only populated when `contract_line_details` is passed in the `include` parameter.
    """
    contract_number: str
    """
    A unique user-provided contract number e.g. C-2026-0001.
    """
    contract_value_details: ContractValueDetails
    """
    The computed total value of all contract lines.
    """
    created: str
    """
    Timestamp of when the object was created.
    """
    currency: str
    """
    The currency of the contract.
    """
    customer: str
    """
    The ID of the customer associated with the contract.
    """
    id: str
    """
    The ID of the contract object.
    """
    license_quantities: List[LicenseQuantity]
    """
    The license quantities of the contract. Only populated when `license_quantities` is passed in the `include` parameter.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of key-value pairs that you can attach to an object.
    """
    object: Literal["v2.billing.contract"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    one_time_fees: Optional[List[OneTimeFee]]
    """
    The one-time fees of the contract. Only populated when `one_time_fees` is passed in the `include` parameter.
    """
    pricing_lines: List[PricingLine]
    """
    The pricing lines of the contract. Only populated when `pricing_lines` is passed in the `include` parameter.
    """
    pricing_overrides: List[PricingOverride]
    """
    The pricing overrides of the contract. Only populated when `pricing_overrides` is passed in the `include` parameter.
    """
    status: Literal["active", "canceled", "draft", "ended"]
    """
    The current status of the contract.
    """
    status_details: StatusDetails
    """
    Information about the contract status transitions.
    """
    _inner_class_types = {
        "billing_settings": BillingSettings,
        "contract_line_details": ContractLineDetail,
        "contract_value_details": ContractValueDetails,
        "license_quantities": LicenseQuantity,
        "one_time_fees": OneTimeFee,
        "pricing_lines": PricingLine,
        "pricing_overrides": PricingOverride,
        "status_details": StatusDetails,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class ContractUpdateParams(TypedDict):
    include: NotRequired[
        List[
            Literal[
                "contract_line_details",
                "license_quantities",
                "one_time_fees",
                "pricing_lines",
                "pricing_overrides",
            ]
        ]
    ]
    """
    Additional fields to include in the response.
    """
    license_quantity_actions: NotRequired[
        List["ContractUpdateParamsLicenseQuantityAction"]
    ]
    """
    Schema-only: License quantity actions (implementation to follow).
    """
    pricing_line_actions: NotRequired[
        List["ContractUpdateParamsPricingLineAction"]
    ]
    """
    Pricing line actions to apply.
    """
    pricing_override_actions: NotRequired[
        List["ContractUpdateParamsPricingOverrideAction"]
    ]
    """
    Pricing override actions to apply.
    """


class ContractUpdateParamsLicenseQuantityAction(TypedDict):
    effective_at: "ContractUpdateParamsLicenseQuantityActionEffectiveAt"
    """
    The effective at for the license quantity action.
    """
    license_pricing_id: NotRequired[str]
    """
    The ID of the license pricing.
    """
    license_pricing_lookup_key: NotRequired[str]
    """
    The lookup key for the license pricing.
    """
    license_pricing_type: Literal["license_fee", "price"]
    """
    The type of the license pricing.
    """
    pricing_line: NotRequired[str]
    """
    The pricing line ID for the license quantity action.
    """
    pricing_line_lookup_key: NotRequired[str]
    """
    The pricing line lookup key for the license quantity action.
    """
    set: NotRequired["ContractUpdateParamsLicenseQuantityActionSet"]
    """
    The set quantity for the license quantity action.
    """
    type: Literal["set"]
    """
    The type of the license quantity action.
    """


class ContractUpdateParamsLicenseQuantityActionEffectiveAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp for the effective at.
    """
    type: Literal["timestamp"]
    """
    The type of the effective at.
    """


class ContractUpdateParamsLicenseQuantityActionSet(TypedDict):
    quantity: int
    """
    The quantity to set.
    """


class ContractUpdateParamsPricingLineAction(TypedDict):
    add: NotRequired["ContractUpdateParamsPricingLineActionAdd"]
    """
    Parameters for adding a pricing line.
    """
    remove: NotRequired["ContractUpdateParamsPricingLineActionRemove"]
    """
    Parameters for removing a pricing line.
    """
    type: Literal["add", "remove", "update"]
    """
    The type of pricing line action.
    """
    update: NotRequired["ContractUpdateParamsPricingLineActionUpdate"]
    """
    Parameters for updating a pricing line.
    """


class ContractUpdateParamsPricingLineActionAdd(TypedDict):
    ends_at: "ContractUpdateParamsPricingLineActionAddEndsAt"
    """
    The end time for the pricing line.
    """
    lookup_key: NotRequired[str]
    """
    A lookup key for the pricing line.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Metadata for the pricing line.
    """
    pricing: "ContractUpdateParamsPricingLineActionAddPricing"
    """
    The pricing configuration for the pricing line.
    """
    starts_at: "ContractUpdateParamsPricingLineActionAddStartsAt"
    """
    The start time for the pricing line.
    """


class ContractUpdateParamsPricingLineActionAddEndsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item ends.
    """
    type: Literal["billing_period_end", "timestamp"]
    """
    The type of end time to apply.
    """


class ContractUpdateParamsPricingLineActionAddPricing(TypedDict):
    price_details: NotRequired[
        "ContractUpdateParamsPricingLineActionAddPricingPriceDetails"
    ]
    """
    V1 price details. Required if `type` is `price`.
    """
    type: Literal["price"]
    """
    The type of pricing.
    """


class ContractUpdateParamsPricingLineActionAddPricingPriceDetails(TypedDict):
    price: str
    """
    The ID of the V1 price.
    """
    quantity: NotRequired[int]
    """
    The quantity for the price. Only applicable for licensed prices.
    """


class ContractUpdateParamsPricingLineActionAddStartsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item starts.
    """
    type: Literal["billing_period_start", "timestamp"]
    """
    The type of start time to apply.
    """


class ContractUpdateParamsPricingLineActionRemove(TypedDict):
    id: str
    """
    The ID of the pricing line to remove.
    """


class ContractUpdateParamsPricingLineActionUpdate(TypedDict):
    ends_at: NotRequired["ContractUpdateParamsPricingLineActionUpdateEndsAt"]
    """
    The updated end time for the pricing line.
    """
    id: str
    """
    The ID of the pricing line.
    """
    starts_at: NotRequired[
        "ContractUpdateParamsPricingLineActionUpdateStartsAt"
    ]
    """
    The updated start time for the pricing line.
    """


class ContractUpdateParamsPricingLineActionUpdateEndsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item ends.
    """
    type: Literal["billing_period_end", "timestamp"]
    """
    The type of end time to apply.
    """


class ContractUpdateParamsPricingLineActionUpdateStartsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item starts.
    """
    type: Literal["billing_period_start", "timestamp"]
    """
    The type of start time to apply.
    """


class ContractUpdateParamsPricingOverrideAction(TypedDict):
    add: NotRequired["ContractUpdateParamsPricingOverrideActionAdd"]
    """
    Parameters for adding a pricing override.
    """
    remove: NotRequired["ContractUpdateParamsPricingOverrideActionRemove"]
    """
    Parameters for removing a pricing override.
    """
    type: Literal["add", "remove", "update"]
    """
    The type of pricing override action.
    """
    update: NotRequired["ContractUpdateParamsPricingOverrideActionUpdate"]
    """
    Parameters for updating a pricing override.
    """


class ContractUpdateParamsPricingOverrideActionAdd(TypedDict):
    ends_at: "ContractUpdateParamsPricingOverrideActionAddEndsAt"
    """
    The end time for the pricing override.
    """
    lookup_key: NotRequired[str]
    """
    A lookup key for the pricing override.
    """
    multiplier: NotRequired[
        "ContractUpdateParamsPricingOverrideActionAddMultiplier"
    ]
    """
    A multiplier override to add.
    """
    overwrite_price: NotRequired[
        "ContractUpdateParamsPricingOverrideActionAddOverwritePrice"
    ]
    """
    An overwrite price override to add.
    """
    priority: int
    """
    The priority for the pricing override. The highest priority is 0 and the lowest is 100.
    """
    starts_at: "ContractUpdateParamsPricingOverrideActionAddStartsAt"
    """
    The start time for the pricing override.
    """
    type: Literal["multiplier", "overwrite_price"]
    """
    The type of pricing override to add.
    """


class ContractUpdateParamsPricingOverrideActionAddEndsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item ends.
    """
    type: Literal["billing_period_end", "timestamp"]
    """
    The type of end time to apply.
    """


class ContractUpdateParamsPricingOverrideActionAddMultiplier(TypedDict):
    criteria: List[
        "ContractUpdateParamsPricingOverrideActionAddMultiplierCriterion"
    ]
    """
    Criteria determining which rates the multiplier applies to.
    """
    factor: str
    """
    The multiplier factor, represented as a decimal string. e.g. "0.8" for a 20% reduction.
    """


class ContractUpdateParamsPricingOverrideActionAddMultiplierCriterion(
    TypedDict,
):
    billable_item_ids: List[str]
    """
    Filter by billable item IDs.
    """
    billable_item_lookup_keys: List[str]
    """
    Filter by billable item lookup keys.
    """
    billable_item_types: List[Literal["licensed", "metered"]]
    """
    Filter by billable item type.
    """
    metadata_conditions: List[
        "ContractUpdateParamsPricingOverrideActionAddMultiplierCriterionMetadataCondition"
    ]
    """
    Filter by metadata conditions.
    """
    rate_card_ids: List[str]
    """
    Filter by rate card IDs. Only applicable for `multiplier` overrides.
    """
    type: Literal["exclude", "include"]
    """
    Whether to include or exclude items matching these criteria.
    """


class ContractUpdateParamsPricingOverrideActionAddMultiplierCriterionMetadataCondition(
    TypedDict,
):
    all_of: List[
        "ContractUpdateParamsPricingOverrideActionAddMultiplierCriterionMetadataConditionAllOf"
    ]
    """
    All of these key-value conditions must match.
    """


class ContractUpdateParamsPricingOverrideActionAddMultiplierCriterionMetadataConditionAllOf(
    TypedDict,
):
    key: str
    """
    The metadata key.
    """
    value: str
    """
    The metadata value.
    """


class ContractUpdateParamsPricingOverrideActionAddOverwritePrice(TypedDict):
    price: str
    """
    The ID of the V1 price to overwrite.
    """
    tiering_mode: NotRequired[Literal["graduated", "volume"]]
    """
    Defines whether the tiered price should be graduated or volume-based.
    """
    tiers: List[
        "ContractUpdateParamsPricingOverrideActionAddOverwritePriceTier"
    ]
    """
    Each element represents a pricing tier.
    """
    unit_amount: NotRequired[str]
    """
    The per-unit amount to be charged, represented as a decimal string in minor currency units.
    """


class ContractUpdateParamsPricingOverrideActionAddOverwritePriceTier(
    TypedDict
):
    flat_amount: NotRequired[str]
    """
    Price for the entire tier, represented as a decimal string in minor currency units.
    """
    unit_amount: NotRequired[str]
    """
    Per-unit price for units included in this tier, represented as a decimal string in minor currency units.
    """
    up_to_decimal: NotRequired[Decimal]
    """
    Up to and including this quantity will be contained in the tier.
    """
    up_to_inf: NotRequired[Literal["inf"]]
    """
    No upper bound to this tier.
    """


class ContractUpdateParamsPricingOverrideActionAddStartsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item starts.
    """
    type: Literal["billing_period_start", "timestamp"]
    """
    The type of start time to apply.
    """


class ContractUpdateParamsPricingOverrideActionRemove(TypedDict):
    id: str
    """
    The ID of the pricing override to remove.
    """


class ContractUpdateParamsPricingOverrideActionUpdate(TypedDict):
    ends_at: NotRequired[
        "ContractUpdateParamsPricingOverrideActionUpdateEndsAt"
    ]
    """
    The updated end time for the pricing override.
    """
    id: str
    """
    The ID of the pricing override.
    """
    starts_at: NotRequired[
        "ContractUpdateParamsPricingOverrideActionUpdateStartsAt"
    ]
    """
    The updated start time for the pricing override.
    """


class ContractUpdateParamsPricingOverrideActionUpdateEndsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item ends.
    """
    type: Literal["billing_period_end", "timestamp"]
    """
    The type of end time to apply.
    """


class ContractUpdateParamsPricingOverrideActionUpdateStartsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item starts.
    """
    type: Literal["billing_period_start", "timestamp"]
    """
    The type of start time to apply.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._stripe_object import UntypedStripeObject
from stripe.v2._amount import AmountParam
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class ContractCreateParams(TypedDict):
    billing_settings: NotRequired["ContractCreateParamsBillingSettings"]
    """
    The billing settings for the contract.
    """
    contract_lines: List["ContractCreateParamsContractLine"]
    """
    A list of contract lines to create with the contract.
    """
    contract_number: str
    """
    A unique user-provided contract number e.g. C-2026-0001.
    """
    currency: str
    """
    Currency of the contract.
    """
    include: NotRequired[
        List[
            Literal[
                "contract_line_details",
                "license_quantities",
                "one_time_fees",
                "pricing_lines",
                "pricing_overrides",
            ]
        ]
    ]
    """
    Additional fields to include in the response.
    """
    license_quantity_actions: List["ContractCreateParamsLicenseQuantityAction"]
    """
    A list of license quantity actions to create with the contract.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of key-value pairs that you can attach to an object.
    """
    one_time_fees: NotRequired[List["ContractCreateParamsOneTimeFee"]]
    """
    A list of one-time fees to create with the contract. Each fee is billed as individual invoice items per its bill_schedule.
    """
    pricing_lines: List["ContractCreateParamsPricingLine"]
    """
    A list of pricing lines to create with the contract.
    """
    pricing_overrides: List["ContractCreateParamsPricingOverride"]
    """
    A list of pricing overrides to create with the contract.
    """


class ContractCreateParamsBillingSettings(TypedDict):
    contract_billing_details: NotRequired[
        "ContractCreateParamsBillingSettingsContractBillingDetails"
    ]
    """
    Billing settings details for the contract.
    """


class ContractCreateParamsBillingSettingsContractBillingDetails(TypedDict):
    bill_settings_details: NotRequired[
        "ContractCreateParamsBillingSettingsContractBillingDetailsBillSettingsDetails"
    ]
    """
    The bill settings details.
    """
    billing_profile_details: "ContractCreateParamsBillingSettingsContractBillingDetailsBillingProfileDetails"
    """
    The billing profile details.
    """
    collection_settings_details: "ContractCreateParamsBillingSettingsContractBillingDetailsCollectionSettingsDetails"
    """
    The collection settings details.
    """


class ContractCreateParamsBillingSettingsContractBillingDetailsBillSettingsDetails(
    TypedDict,
):
    calculation: NotRequired[
        "ContractCreateParamsBillingSettingsContractBillingDetailsBillSettingsDetailsCalculation"
    ]
    """
    Calculation settings.
    """
    invoice: NotRequired[
        "ContractCreateParamsBillingSettingsContractBillingDetailsBillSettingsDetailsInvoice"
    ]
    """
    Invoice settings.
    """


class ContractCreateParamsBillingSettingsContractBillingDetailsBillSettingsDetailsCalculation(
    TypedDict,
):
    tax: NotRequired[
        "ContractCreateParamsBillingSettingsContractBillingDetailsBillSettingsDetailsCalculationTax"
    ]
    """
    Tax calculation settings.
    """


class ContractCreateParamsBillingSettingsContractBillingDetailsBillSettingsDetailsCalculationTax(
    TypedDict,
):
    type: Literal["automatic", "manual"]
    """
    The type of tax calculation.
    """


class ContractCreateParamsBillingSettingsContractBillingDetailsBillSettingsDetailsInvoice(
    TypedDict,
):
    time_until_due: NotRequired[
        "ContractCreateParamsBillingSettingsContractBillingDetailsBillSettingsDetailsInvoiceTimeUntilDue"
    ]
    """
    The number of time units before the invoice is past due.
    """


class ContractCreateParamsBillingSettingsContractBillingDetailsBillSettingsDetailsInvoiceTimeUntilDue(
    TypedDict,
):
    interval: Literal["day", "month", "week", "year"]
    """
    The interval unit.
    """
    interval_count: int
    """
    The number of intervals.
    """


class ContractCreateParamsBillingSettingsContractBillingDetailsBillingProfileDetails(
    TypedDict,
):
    customer: str
    """
    The customer who pays for the contract invoice.
    """
    default_payment_method: NotRequired[str]
    """
    The default payment method for the contract.
    """


class ContractCreateParamsBillingSettingsContractBillingDetailsCollectionSettingsDetails(
    TypedDict,
):
    collection_method: Literal["charge_automatically", "send_invoice"]
    """
    The collection method.
    """
    payment_method_configuration: NotRequired[str]
    """
    The payment method configuration.
    """


class ContractCreateParamsContractLine(TypedDict):
    ends_at: "ContractCreateParamsContractLineEndsAt"
    """
    Timestamp to indicate when the contract line ends.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of key-value pairs that you can attach to an object.
    """
    overrides: List["ContractCreateParamsContractLineOverride"]
    """
    List of overrides. Later overrides in the list override earlier ones.
    """
    pricing: "ContractCreateParamsContractLinePricing"
    """
    The pricing configuration for the contract line.
    """
    starts_at: "ContractCreateParamsContractLineStartsAt"
    """
    Timestamp to indicate when the contract line starts.
    """


class ContractCreateParamsContractLineEndsAt(TypedDict):
    timestamp: str
    """
    The timestamp when the item ends.
    """


class ContractCreateParamsContractLineOverride(TypedDict):
    ends_at: "ContractCreateParamsContractLineOverrideEndsAt"
    """
    Timestamp to indicate when the override ends.
    """
    service_action: NotRequired[
        "ContractCreateParamsContractLineOverrideServiceAction"
    ]
    """
    Service action override parameters. Required if `type` is `service_action`.
    """
    starts_at: "ContractCreateParamsContractLineOverrideStartsAt"
    """
    Timestamp to indicate when the override starts.
    """
    type: Literal["service_action"]
    """
    The type of the override.
    """


class ContractCreateParamsContractLineOverrideEndsAt(TypedDict):
    timestamp: str
    """
    The timestamp when the item ends.
    """


class ContractCreateParamsContractLineOverrideServiceAction(TypedDict):
    add: NotRequired[
        "ContractCreateParamsContractLineOverrideServiceActionAdd"
    ]
    """
    Parameters for adding a new service action.
    """
    replace: NotRequired[
        "ContractCreateParamsContractLineOverrideServiceActionReplace"
    ]
    """
    Parameters for replacing an existing service action.
    """
    type: Literal["add", "replace"]
    """
    The type of service action override.
    """


class ContractCreateParamsContractLineOverrideServiceActionAdd(TypedDict):
    credit_grant: NotRequired[
        "ContractCreateParamsContractLineOverrideServiceActionAddCreditGrant"
    ]
    """
    Details for the credit grant. Required if `type` is `credit_grant`.
    """
    service_interval: Literal["day", "month", "week", "year"]
    """
    The interval for assessing service.
    """
    service_interval_count: int
    """
    The length of the interval for assessing service.
    """
    type: Literal["credit_grant"]
    """
    The type of the service action.
    """


class ContractCreateParamsContractLineOverrideServiceActionAddCreditGrant(
    TypedDict,
):
    amount: "ContractCreateParamsContractLineOverrideServiceActionAddCreditGrantAmount"
    """
    The amount of the credit grant.
    """
    applicability_config: "ContractCreateParamsContractLineOverrideServiceActionAddCreditGrantApplicabilityConfig"
    """
    Defines the scope where the credit grant is applicable.
    """
    category: NotRequired[Literal["paid", "promotional"]]
    """
    The category of the credit grant.
    """
    expiry_config: "ContractCreateParamsContractLineOverrideServiceActionAddCreditGrantExpiryConfig"
    """
    The expiry configuration for the credit grant.
    """
    name: str
    """
    A descriptive name.
    """
    priority: NotRequired[int]
    """
    The desired priority for applying this credit grant. The highest priority is 0 and the lowest is 100.
    """


class ContractCreateParamsContractLineOverrideServiceActionAddCreditGrantAmount(
    TypedDict,
):
    monetary: NotRequired[AmountParam]
    """
    The monetary amount of the credit grant. Required if `type` is `monetary`.
    """
    type: Literal["monetary"]
    """
    The type of the credit grant amount.
    """


class ContractCreateParamsContractLineOverrideServiceActionAddCreditGrantApplicabilityConfig(
    TypedDict,
):
    scope: "ContractCreateParamsContractLineOverrideServiceActionAddCreditGrantApplicabilityConfigScope"
    """
    The applicability scope of the credit grant.
    """


class ContractCreateParamsContractLineOverrideServiceActionAddCreditGrantApplicabilityConfigScope(
    TypedDict,
):
    billable_items: NotRequired[List[str]]
    """
    The billable items to apply the credit grant to.
    """
    price_type: NotRequired[Literal["metered"]]
    """
    The price type that credit grants can apply to.
    """


class ContractCreateParamsContractLineOverrideServiceActionAddCreditGrantExpiryConfig(
    TypedDict,
):
    type: Literal["end_of_service_period"]
    """
    The type of the expiry configuration.
    """


class ContractCreateParamsContractLineOverrideServiceActionReplace(TypedDict):
    credit_grant: NotRequired[
        "ContractCreateParamsContractLineOverrideServiceActionReplaceCreditGrant"
    ]
    """
    Details for the credit grant. Required if `type` is `credit_grant`.
    """
    id: NotRequired[str]
    """
    The ID of the service action to replace.
    """
    lookup_key: NotRequired[str]
    """
    The lookup key for the service action to replace.
    """
    service_interval: Literal["day", "month", "week", "year"]
    """
    The interval for assessing service.
    """
    service_interval_count: int
    """
    The length of the interval for assessing service.
    """
    type: Literal["credit_grant"]
    """
    The type of the service action.
    """


class ContractCreateParamsContractLineOverrideServiceActionReplaceCreditGrant(
    TypedDict,
):
    amount: "ContractCreateParamsContractLineOverrideServiceActionReplaceCreditGrantAmount"
    """
    The amount of the credit grant.
    """
    applicability_config: "ContractCreateParamsContractLineOverrideServiceActionReplaceCreditGrantApplicabilityConfig"
    """
    Defines the scope where the credit grant is applicable.
    """
    category: NotRequired[Literal["paid", "promotional"]]
    """
    The category of the credit grant.
    """
    expiry_config: "ContractCreateParamsContractLineOverrideServiceActionReplaceCreditGrantExpiryConfig"
    """
    The expiry configuration for the credit grant.
    """
    name: str
    """
    A descriptive name.
    """
    priority: NotRequired[int]
    """
    The desired priority for applying this credit grant. The highest priority is 0 and the lowest is 100.
    """


class ContractCreateParamsContractLineOverrideServiceActionReplaceCreditGrantAmount(
    TypedDict,
):
    monetary: NotRequired[AmountParam]
    """
    The monetary amount of the credit grant. Required if `type` is `monetary`.
    """
    type: Literal["monetary"]
    """
    The type of the credit grant amount.
    """


class ContractCreateParamsContractLineOverrideServiceActionReplaceCreditGrantApplicabilityConfig(
    TypedDict,
):
    scope: "ContractCreateParamsContractLineOverrideServiceActionReplaceCreditGrantApplicabilityConfigScope"
    """
    The applicability scope of the credit grant.
    """


class ContractCreateParamsContractLineOverrideServiceActionReplaceCreditGrantApplicabilityConfigScope(
    TypedDict,
):
    billable_items: NotRequired[List[str]]
    """
    The billable items to apply the credit grant to.
    """
    price_type: NotRequired[Literal["metered"]]
    """
    The price type that credit grants can apply to.
    """


class ContractCreateParamsContractLineOverrideServiceActionReplaceCreditGrantExpiryConfig(
    TypedDict,
):
    type: Literal["end_of_service_period"]
    """
    The type of the expiry configuration.
    """


class ContractCreateParamsContractLineOverrideStartsAt(TypedDict):
    timestamp: str
    """
    The timestamp when the item starts.
    """


class ContractCreateParamsContractLinePricing(TypedDict):
    pass


class ContractCreateParamsContractLineStartsAt(TypedDict):
    timestamp: str
    """
    The timestamp when the item starts.
    """


class ContractCreateParamsLicenseQuantityAction(TypedDict):
    effective_at: "ContractCreateParamsLicenseQuantityActionEffectiveAt"
    """
    The effective at for the license quantity action.
    """
    license_pricing_id: NotRequired[str]
    """
    The ID of the license pricing.
    """
    license_pricing_lookup_key: NotRequired[str]
    """
    The lookup key for the license pricing.
    """
    license_pricing_type: Literal["license_fee", "price"]
    """
    The type of the license pricing.
    """
    pricing_line: NotRequired[str]
    """
    The pricing line for the license quantity action.
    """
    set: NotRequired["ContractCreateParamsLicenseQuantityActionSet"]
    """
    The set quantity for the license quantity action.
    """
    type: Literal["set"]
    """
    The type of the license quantity action.
    """


class ContractCreateParamsLicenseQuantityActionEffectiveAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp for the effective at.
    """
    type: Literal["timestamp"]
    """
    The type of the effective at.
    """


class ContractCreateParamsLicenseQuantityActionSet(TypedDict):
    quantity: int
    """
    The quantity to set.
    """


class ContractCreateParamsOneTimeFee(TypedDict):
    bill_schedule: List["ContractCreateParamsOneTimeFeeBillSchedule"]
    """
    The bill schedule for the fee. Each entry produces an individual invoice item billed at `bill_at`.
    """
    billable_item_type: Literal["product"]
    """
    The type of billable item that this fee references.
    """
    product_details: NotRequired[
        "ContractCreateParamsOneTimeFeeProductDetails"
    ]
    """
    Details for a product billable target. Required when `billable_item_type` is `product`.
    """


class ContractCreateParamsOneTimeFeeBillSchedule(TypedDict):
    bill_at: "ContractCreateParamsOneTimeFeeBillScheduleBillAt"
    """
    When this entry should be billed.
    """
    value: int
    """
    The amount to bill for this entry, in the smallest currency unit.
    """


class ContractCreateParamsOneTimeFeeBillScheduleBillAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The datetime at which the entry should be billed. Required if `type` is `datetime`.
    """
    type: Literal["contract_start", "datetime"]
    """
    The type of the bill_at.
    """


class ContractCreateParamsOneTimeFeeProductDetails(TypedDict):
    product: str
    """
    The ID of the v1 Product.
    """


class ContractCreateParamsPricingLine(TypedDict):
    ends_at: "ContractCreateParamsPricingLineEndsAt"
    """
    When the pricing line ends.
    """
    lookup_key: NotRequired[str]
    """
    A user-provided lookup key to reference this pricing line.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of key-value pairs that you can attach to an object.
    """
    pricing: "ContractCreateParamsPricingLinePricing"
    """
    The pricing configuration for the pricing line.
    """
    starts_at: "ContractCreateParamsPricingLineStartsAt"
    """
    When the pricing line starts.
    """


class ContractCreateParamsPricingLineEndsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item ends. Required if `type` is `timestamp`.
    """
    type: Literal["contract_end", "timestamp"]
    """
    The type of the ends_at.
    """


class ContractCreateParamsPricingLinePricing(TypedDict):
    price_details: NotRequired[
        "ContractCreateParamsPricingLinePricingPriceDetails"
    ]
    """
    V1 price details. Required if `type` is `price`.
    """
    type: Literal["price"]
    """
    The type of pricing.
    """


class ContractCreateParamsPricingLinePricingPriceDetails(TypedDict):
    price: str
    """
    The ID of the V1 price.
    """
    quantity: NotRequired[int]
    """
    The quantity for the price. Only applicable for licensed prices.
    """


class ContractCreateParamsPricingLineStartsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item starts. Required if `type` is `timestamp`.
    """
    type: Literal["contract_start", "timestamp"]
    """
    The type of the starts_at.
    """


class ContractCreateParamsPricingOverride(TypedDict):
    ends_at: "ContractCreateParamsPricingOverrideEndsAt"
    """
    When the pricing override ends.
    """
    lookup_key: NotRequired[str]
    """
    A user-provided lookup key to reference this pricing override.
    """
    multiplier: NotRequired["ContractCreateParamsPricingOverrideMultiplier"]
    """
    Parameters for a multiplier override. Required if `type` is `multiplier`.
    """
    overwrite_price: NotRequired[
        "ContractCreateParamsPricingOverrideOverwritePrice"
    ]
    """
    Parameters for an overwrite_price override. Required if `type` is `overwrite_price`.
    """
    priority: int
    """
    The priority of this override relative to others. The highest priority is 0 and the lowest is 100.
    """
    starts_at: "ContractCreateParamsPricingOverrideStartsAt"
    """
    When the pricing override starts.
    """
    type: Literal["multiplier", "overwrite_price"]
    """
    The type of pricing override.
    """


class ContractCreateParamsPricingOverrideEndsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item ends. Required if `type` is `timestamp`.
    """
    type: Literal["contract_end", "timestamp"]
    """
    The type of the ends_at.
    """


class ContractCreateParamsPricingOverrideMultiplier(TypedDict):
    criteria: List["ContractCreateParamsPricingOverrideMultiplierCriterion"]
    """
    Criteria determining which rates the multiplier applies to.
    """
    factor: str
    """
    The multiplier factor, represented as a decimal string. e.g. "0.8" for a 20% reduction.
    """


class ContractCreateParamsPricingOverrideMultiplierCriterion(TypedDict):
    billable_item_ids: List[str]
    """
    Filter by billable item IDs.
    """
    billable_item_lookup_keys: List[str]
    """
    Filter by billable item lookup keys.
    """
    billable_item_types: List[Literal["licensed", "metered"]]
    """
    Filter by billable item type.
    """
    metadata_conditions: List[
        "ContractCreateParamsPricingOverrideMultiplierCriterionMetadataCondition"
    ]
    """
    Filter by metadata conditions.
    """
    rate_card_ids: List[str]
    """
    Filter by rate card IDs. Only applicable for `multiplier` overrides.
    """
    type: Literal["exclude", "include"]
    """
    Whether to include or exclude items matching these criteria.
    """


class ContractCreateParamsPricingOverrideMultiplierCriterionMetadataCondition(
    TypedDict,
):
    all_of: List[
        "ContractCreateParamsPricingOverrideMultiplierCriterionMetadataConditionAllOf"
    ]
    """
    All of these key-value conditions must match.
    """


class ContractCreateParamsPricingOverrideMultiplierCriterionMetadataConditionAllOf(
    TypedDict,
):
    key: str
    """
    The metadata key.
    """
    value: str
    """
    The metadata value.
    """


class ContractCreateParamsPricingOverrideOverwritePrice(TypedDict):
    price: str
    """
    The ID of the V1 price to overwrite.
    """
    tiering_mode: NotRequired[Literal["graduated", "volume"]]
    """
    Defines whether the tiered price should be graduated or volume-based.
    """
    tiers: NotRequired[
        List["ContractCreateParamsPricingOverrideOverwritePriceTier"]
    ]
    """
    Each element represents a pricing tier.
    """
    unit_amount: NotRequired[str]
    """
    The per-unit amount to be charged, represented as a decimal string in minor currency units.
    """


class ContractCreateParamsPricingOverrideOverwritePriceTier(TypedDict):
    flat_amount: NotRequired[str]
    """
    Price for the entire tier, represented as a decimal string in minor currency units.
    """
    unit_amount: NotRequired[str]
    """
    Per-unit price for units included in this tier, represented as a decimal string in minor currency units.
    """
    up_to_decimal: NotRequired[Decimal]
    """
    Up to and including this quantity will be contained in the tier.
    """
    up_to_inf: NotRequired[Literal["inf"]]
    """
    No upper bound to this tier.
    """


class ContractCreateParamsPricingOverrideStartsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item starts. Required if `type` is `timestamp`.
    """
    type: Literal["contract_start", "timestamp"]
    """
    The type of the starts_at.
    """
